==> lstdataprocess128-1.3.0.tar.gz: lstdataprocess128-1.3.0/src/lstdpp128/apps/cameo/CameoLiveCoincidence.cpp <==
/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cstdlib>
#include <fstream>
#include <sstream>
#include <cstring>
#include <algorithm>
#include <time.h>
#include <cstdlib> //std::system
#include <cameo/cameo.h>
#include <boost/property_tree/json_parser.hpp>

#include "../../algorithms/nomad/Coincidence.h"
#include "../../BaseReader.h"
#include "../../Event.h"
#include "../../EventData.h"
#include "../../EventBlockArray.h"
#include "../../EventBlockArrayAlgorithm.h"
#include "Utils.h"

using namespace std;
using namespace lstdpp128;
using namespace cameo;
using namespace boost;

int32_t MAX_BLOCK_SIZE = 0;
int32_t BUFFER_SIZE = 1 << 12;

int percentage(int a, int b) {
	return 100.0f * (float)a / (float)b;
}

void printRate(double * rate, ostream& out) {

	for (int i = 0; i < 7; ++i) {
		out << setw(12) << rate[i] << " ";
	}
	out << endl;
}

void createFile(const std::string& fileName) {
	std::ofstream outDataFile;
	outDataFile.open(fileName.c_str());
	outDataFile.close();
}

void checkBoards() {

	// Print the columns of the event data.
	BoardIterator b;
	while (!b.end()) {
		if (!boardHasEnergy((*b).boardType)) {
			cerr << "Warning: board " << BoardTypeName[(*b).boardType] << " has no energy data." << endl;
		}
		++b;
	}
}

int main(int argc, char * argv[]) {

	application::This::init(argc, argv);

	{
		// The JSON parameters are in the first argument
		string jsonParameters(argv[1]);

		cout << "parsing " << jsonParameters << endl;

		property_tree::ptree parameters;

		stringstream ss;
		ss << jsonParameters;

		read_json(ss, parameters);

		// Reading the parameters
		string numor = parameters.get<string>("numor");
		time_type dT = parameters.get<time_type>("dT");
		time_type cleanDT = parameters.get<time_type>("cleanDT");

		vector<int16_t> detectorADCList;
		property_tree::ptree detectorChannels = parameters.get_child("detectorChannels");

		for (property_tree::ptree::const_iterator c = detectorChannels.begin(); c != detectorChannels.end(); ++c) {
			detectorADCList.push_back(c->second.get<int16_t>(""));
		}

		// Do not use the coincidence matrix in live mode
		int32_t matrixResolution = 0; //parameters.get<int32_t>("matrixResolution");
		int32_t maxBlocks = parameters.get<int32_t>("maxBlocks");
		BUFFER_SIZE = parameters.get<int32_t>("blockSize");
		int32_t resultPeriod = parameters.get<int32_t>("refreshTime");

		property_tree::ptree context = parameters.get_child("context");

		readListModeContextFromPropertyTree(listModeContext, context);

		cout << listModeContext << endl;

		MAX_BLOCK_SIZE = BUFFER_SIZE * 2;

		string outFileName = "/users/data/" + numor + ".lsto";
		string logFileName = outFileName + ".log";
		ofstream log(logFileName.c_str());

		log << "analyzing " << numor << ", dT " << dT << ", clean dT " << cleanDT << ", max blocks " << maxBlocks << ", block size " << BUFFER_SIZE << endl;

		// connect to the Nomad server
		auto_ptr<application::Instance> nomad = application::This::connectToStarter();

		cout << "connected to " << *nomad << endl;

		// create the subscriber
		auto_ptr<application::Subscriber> subscriber = application::Subscriber::create(*nomad, "dpp_publisher");

		cout << "synchronized with " << *subscriber << endl;

		// create the publisher
		auto_ptr<application::Publisher> publisher = application::Publisher::create("dpp_coincidence_results");

		cout << "created publisher " << *publisher << endl;

		// Check boards.
		checkBoards();

		// algorithm
		EventBlockArrayAlgorithm<Coincidence> algorithm;
		algorithm.processor().init(0, 0, 0, 0, matrixResolution, detectorADCList, dT, cleanDT, MAX_BLOCK_SIZE, "");

		// creating the events
		EventBlockArray blocks(MAX_BLOCK_SIZE, maxBlocks);

		// set the application running
		application::This::setRunning();

		time_t t1 = time(0);
		time_t last = time(0);

		string data;
		while (subscriber->receiveBinary(data)) {
			int32_t blockSize = data.size() / sizeof(int32_t);
			int32_t * block = const_cast<int32_t *>(reinterpret_cast<const int32_t *>(data.c_str()));

			// processing block
			algorithm.process(block, blockSize, blocks);

			time_t current = time(0);
			if (current - last > resultPeriod) {
				cout << "#events read " << algorithm.processor().numberOfGoodEvents() << endl;

				// publish results
				publisher->send(algorithm.processor().getChannelRates(), algorithm.processor().getTotalNumberOfChannels());
				publisher->send(algorithm.processor().getCoincidenceRates(), algorithm.processor().getNumberOfCoincidenceRates());

				last = current;
			}
		}

		// publish results at the end
		publisher->send(algorithm.processor().getChannelRates(), algorithm.processor().getTotalNumberOfChannels());
		publisher->send(algorithm.processor().getCoincidenceRates(), algorithm.processor().getNumberOfCoincidenceRates());

		time_t t2 = time(0);

		algorithm.finish(blocks);

		log << "#events read         = " << algorithm.processor().numberOfGoodEvents() << " (" << ((float)algorithm.processor().numberOfGoodEvents() / (float)(t2 - t1)) << " event/s)" << endl;
		log << "#events bad          = " << algorithm.processor().numberOfBadEvents() << " (" << percentage(algorithm.processor().numberOfBadEvents(), algorithm.processor().numberOfGoodEvents()) << "%)" << endl;
		log << "#events not inserted = " << algorithm.processor().numberOfEventsNotInserted() << " (" << percentage(algorithm.processor().numberOfEventsNotInserted(), algorithm.processor().numberOfGoodEvents()) << "%)" << endl;

		log.close();
	}

	application::This::terminate();

	return EXIT_SUCCESS;
}

==> lstdataprocess128-1.3.2.tar.gz: lstdataprocess128-1.3.2/src/lstdpp128/apps/cameo/CameoLiveCoincidence.cpp <==
/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://www.osor.eu/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cstdlib>
#include <fstream>
#include <sstream>
#include <cstring>
#include <algorithm>
#include <time.h>
#include <cstdlib> //std::system
#include <cameo/cameo.h>
#include <boost/property_tree/json_parser.hpp>

#include "../../algorithms/nomad/Coincidence.h"
#include "../../BaseReader.h"
#include "../../Event.h"
#include "../../EventData.h"
#include "../../EventBlockArray.h"
#include "../../EventBlockArrayAlgorithm.h"
#include "Utils.h"

using namespace std;
using namespace lstdpp128;
using namespace cameo;
using namespace boost;

int32_t MAX_BLOCK_SIZE = 0;
int32_t BUFFER_SIZE = 1 << 12;

int percentage(int a, int b) {
	return 100.0f * (float)a / (float)b;
}

void printRate(double * rate, ostream& out) {

	for (int i = 0; i < 7; ++i) {
		out << setw(12) << rate[i] << " ";
	}
	out << endl;
}

void createFile(const std::string& fileName) {
	std::ofstream outDataFile;
	outDataFile.open(fileName.c_str());
	outDataFile.close();
}

void checkBoards() {

	// Print the columns of the event data.
	BoardIterator b;
	while (!b.end()) {
		if (!boardHasEnergy((*b).boardType)) {
			cerr << "Warning: board " << BoardTypeName[(*b).boardType] << " has no energy data." << endl;
		}
		++b;
	}
}

int main(int argc, char * argv[]) {

	application::This::init(argc, argv);

	{
		// The JSON parameters are in the first argument
		string jsonParameters(argv[1]);

		cout << "parsing " << jsonParameters << endl;

		property_tree::ptree parameters;

		stringstream ss;
		ss << jsonParameters;

		read_json(ss, parameters);

		// Reading the parameters
		string numor = parameters.get<string>("numor");
		time_type dT = parameters.get<time_type>("dT");
		time_type cleanDT = parameters.get<time_type>("cleanDT");

		vector<int16_t> detectorADCList;
		property_tree::ptree detectorChannels = parameters.get_child("detectorChannels");

		for (property_tree::ptree::const_iterator c = detectorChannels.begin(); c != detectorChannels.end(); ++c) {
			detectorADCList.push_back(c->second.get<int16_t>(""));
		}

		// Do not use the coincidence matrix in live mode
		int32_t matrixResolution = 0; //parameters.get<int32_t>("matrixResolution");
		int32_t maxBlocks = parameters.get<int32_t>("maxBlocks");
		BUFFER_SIZE = parameters.get<int32_t>("blockSize");
		int32_t resultPeriod = parameters.get<int32_t>("refreshTime");

		property_tree::ptree context = parameters.get_child("context");

		readListModeContextFromPropertyTree(listModeContext, context);

		cout << listModeContext << endl;

		MAX_BLOCK_SIZE = BUFFER_SIZE * 2;

		string outFileName = "/users/data/" + numor + ".lsto";
		string logFileName = outFileName + ".log";
		ofstream log(logFileName.c_str());

		log << "analyzing " << numor << ", dT " << dT << ", clean dT " << cleanDT << ", max blocks " << maxBlocks << ", block size " << BUFFER_SIZE << endl;

		// connect to the Nomad server
		unique_ptr<application::Instance> nomad = application::This::connectToStarter();

		cout << "connected to " << *nomad << endl;

		// create the subscriber
		unique_ptr<application::Subscriber> subscriber = application::Subscriber::create(*nomad, "dpp_publisher");

		cout << "synchronized with " << *subscriber << endl;

		// create the publisher
		unique_ptr<application::Publisher> publisher = application::Publisher::create("dpp_coincidence_results");

		cout << "created publisher " << *publisher << endl;

		// Check boards.
		checkBoards();

		// algorithm
		EventBlockArrayAlgorithm<Coincidence> algorithm;
		algorithm.processor().init(0, 0, 0, 0, matrixResolution, detectorADCList, dT, cleanDT, MAX_BLOCK_SIZE, "");

		// creating the events
		EventBlockArray blocks(MAX_BLOCK_SIZE, maxBlocks);

		// set the application running
		application::This::setRunning();

		time_t t1 = time(0);
		time_t last = time(0);

		string data;
		while (subscriber->receiveBinary(data)) {
			int32_t blockSize = data.size() / sizeof(int32_t);
			int32_t * block = const_cast<int32_t *>(reinterpret_cast<const int32_t *>(data.c_str()));

			// processing block
			algorithm.process(block, blockSize, blocks);

			time_t current = time(0);
			if (current - last > resultPeriod) {
				cout << "#events read " << algorithm.processor().numberOfGoodEvents() << endl;

				// publish results
				publisher->send(algorithm.processor().getChannelRates(), algorithm.processor().getTotalNumberOfChannels());
				publisher->send(algorithm.processor().getCoincidenceRates(), algorithm.processor().getNumberOfCoincidenceRates());

				last = current;
			}
		}

		// publish results at the end
		publisher->send(algorithm.processor().getChannelRates(), algorithm.processor().getTotalNumberOfChannels());
		publisher->send(algorithm.processor().getCoincidenceRates(), algorithm.processor().getNumberOfCoincidenceRates());

		time_t t2 = time(0);

		algorithm.finish(blocks);

		log << "#events read         = " << algorithm.processor().numberOfGoodEvents() << " (" << ((float)algorithm.processor().numberOfGoodEvents() / (float)(t2 - t1)) << " event/s)" << endl;
		log << "#events bad          = " << algorithm.processor().numberOfBadEvents() << " (" << percentage(algorithm.processor().numberOfBadEvents(), algorithm.processor().numberOfGoodEvents()) << "%)" << endl;
		log << "#events not inserted = " << algorithm.processor().numberOfEventsNotInserted() << " (" << percentage(algorithm.processor().numberOfEventsNotInserted(), algorithm.processor().numberOfGoodEvents()) << "%)" << endl;

		log.close();
	}

	application::This::terminate();

	return EXIT_SUCCESS;
}
